(ns puppetlabs.classifier.reporting)

(def service-id "classifier")

(defn- delta-parent-event
  [delta]
  (if (contains? delta :parent)
    {:type "edit", :what "node_group_parent", :description "edit_node_group_parent"
     :message (str "Change the parent to " (:parent delta))}))

(defn- delta-rule-event
  [delta]
  (if (contains? delta :rule)
    {:type "edit", :what "node_group_rule", :description "edit_node_group_rule"
     :message (str "Change the rule to " (pr-str (:rule delta)))}))

(defn- delta-environment-event
  [delta]
  (if (contains? delta :environment)
    {:type "edit", :what "node_group_environment", :description "edit_node_group_environment"
     :message (str "Change the environment to " (pr-str (:environment delta)))}))

(defn- delta-environment-trumps-event
  [delta]
  (if (contains? delta :environment-trumps)
    {:type "edit", :what "node_group_environment_override", :description "edit_node_group_environment_override"
     :message (str "Change the environment override setting to " (pr-str (:environment-trumps delta)))}))

(defn- delta-name-event
  [delta]
  (if (contains? delta :name)
    {:type "edit", :what "node_group_name", :description "edit_node_group_name"
     :message (str "Change the name to " (pr-str (:name delta)))}))

(defn- delta-description-event
  [delta]
  (if (contains? delta :description)
    {:type "edit", :what "node_group_description", :description "edit_node_group_description"
     :message (str "Change the description to " (pr-str (:description delta)))}))

(defn- add-class-event
  [class]
  {:type "edit" :what "node_group_class"
   :description (str "add_node_group_class_" (name class))
   :message (str "Add the " (pr-str (name class)) " class")})

(defn- delta-classes-events
  [delta group]
  (->> (for [[class params] (:classes delta)
             :let [class-name (name class)
                   old-class (get-in group [:classes class])]]
         (cond
           (and (nil? params) (contains? (:classes group) class))
           [{:type "edit" :what "node_group_class"
             :description (str "delete_node_group_class_" class-name)
             :message (str "Remove the " (pr-str class-name) " class")}]

           (nil? params) nil ; spurious removal

           (empty? params)
           [(add-class-event class-name)]

           :else (let [param-events (for [[param value] params
                                          :let [param-name (name param)]]
                                      (cond
                                        (and (nil? value) (contains? old-class param))
                                        {:type "edit" :what "node_group_class_parameter"
                                         :description (str "delete_node_group_class_parameter_"
                                                           class-name "_"
                                                           param-name)
                                         :message (str "Remove the " (pr-str param-name)
                                                       " parameter from the "
                                                       (pr-str class-name) " class")}

                                        (nil? value) nil

                                        (contains? old-class param)
                                        {:type "edit" :what "node_group_class_parameter"
                                         :description (str "edit_node_group_class_parameter_"
                                                           class-name "_"
                                                           param-name)
                                         :message (str "Change the value of the "
                                                       (pr-str param-name) " parameter of the "
                                                       (pr-str class-name) " class")}

                                        :else ; the param is new
                                        {:type "edit" :what "node_group_class_parameter"
                                         :description (str "add_node_group_class_parameter_"
                                                           class-name "_"
                                                           param-name)
                                         :message (str "Add the " (pr-str (name param))
                                                       " parameter to the "
                                                       (pr-str class-name) " class")}))]
                   (if (contains? (:classes group) class)
                     param-events
                     (conj param-events (add-class-event class-name))))))
    (apply concat)
    (keep identity)))

(defn- delta-variables-events
  [delta group]
  (->> (for [[variable value] (:variables delta)
             :let [var-name (name variable)]]
         (cond
           (and (nil? value) (contains? (:variables group) variable))
           {:type "edit" :what "node_group_variable"
            :description (str "delete_node_group_variable_" var-name)
            :message (str "Remove the " (pr-str var-name) " variable")}

           (nil? value) nil

           (contains? (:variables group) variable)
           {:type "edit" :what "node_group_variable"
            :description (str "edit_node_group_variable_" var-name)
            :message (str "Change the value of the " (pr-str var-name) " variable")}

           :else ; variable is new
           {:type "edit" :what "node_group_variable"
            :description (str "add_node_group_variable_" var-name)
            :message (str "Add the " (pr-str var-name) " variable")}))
    (keep identity)))

(defn delta->events
  [delta group]
  (if (nil? delta)
    [{:type "delete", :what "node_group", :description "delete_node_group"
      :message "Delete the group"}]
    (let [primitive-field-events (juxt delta-parent-event delta-rule-event delta-environment-event
                                       delta-environment-trumps-event delta-name-event delta-description-event)]
      (->> (concat (primitive-field-events delta)
                   (delta-classes-events delta group)
                   (delta-variables-events delta group))
        (keep identity)))))

(defn bundle-besides-events
  [shiro-subject group]
  {:commit
   {:service {:id service-id}
    :subject shiro-subject
    :object {:type "node_groups" :id (-> (:id group) str) :name (:name group)}}})

(defn delta-report
  [shiro-subject delta group]
  (assoc-in (bundle-besides-events shiro-subject group) [:commit :events]
            (delta->events delta group)))

(defn creation-report
  [shiro-subject group]
  (let [initial-events (delta-report shiro-subject (dissoc group :name :id) {})]
    (assoc-in (bundle-besides-events shiro-subject group) [:commit :events]
              (apply vector
                     {:type "create", :what "node_group", :description "create_node_group"
                      :message (str "Create the " (pr-str (:name group)) " group"
                                    " with id " (:id group))}
                     (get-in initial-events [:commit :events])))))

(defn deletion-report
  [shiro-subject group]
  (assoc-in (bundle-besides-events shiro-subject group) [:commit :events]
            [{:type "delete" :what "node_group" :description "delete_node_group"
              :message (str "Delete the " (pr-str (:name group)) " group"
                            " with id " (:id group))}]))
